// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "album_selector.h"
#include "music.h"
#include "tooltip.h"

#include <glibmm/i18n.h>
#include <gtkmm/stock.h>
#include <iomanip>

AlbumView::AlbumView()
	: TreeViewBase(_("Albums"))
{
	m_Column.pack_start(m_AlbumArt, false);
	m_Column.set_cell_data_func(m_AlbumArt, sigc::mem_fun(*this, &AlbumView::AlbumArtDataFunc));

	m_Column.pack_start(m_MusicName);
	m_Column.set_cell_data_func(m_MusicName, sigc::mem_fun(*this, &AlbumView::MusicNameDataFunc));

	get_selection()->set_mode(Gtk::SELECTION_BROWSE);
}

// Scale the album art down to a reasonable size for the treeview.
void AlbumView::AlbumArtDataFunc(Gtk::CellRenderer *renderer,
		const Gtk::TreeIter& iter)
{
	if (Glib::RefPtr<album_t> album =
			Glib::RefPtr<album_t>::cast_dynamic(iter->get_value(m_StoreColumns.m_Data)))
	{
		if (album->art)
		{
			((Gtk::CellRendererPixbuf*)renderer)->property_stock_id().reset_value();
			Glib::RefPtr<Gdk::Pixbuf> scaled_album_art =
				album->art->scale_simple(24, 24, Gdk::INTERP_BILINEAR);
			((Gtk::CellRendererPixbuf*)renderer)->property_pixbuf() = scaled_album_art;
		}
		else
			((Gtk::CellRendererPixbuf*)renderer)->property_stock_id() =
				Gtk::Stock::MISSING_IMAGE.id;
		renderer->set_visible();
	}
	else
		renderer->set_visible(false);
}

// Set the text that is shown in the music treeview.
void AlbumView::MusicNameDataFunc(Gtk::CellRenderer *renderer,
		const Gtk::TreeIter& iter)
{
	Glib::RefPtr<mtp_data_t> mtp_data = (*iter)[m_StoreColumns.m_Data];
	if (Glib::RefPtr<track_t> track = Glib::RefPtr<track_t>::cast_dynamic(mtp_data))
	{
		if (track->TrackNumber())
			((Gtk::CellRendererText*)renderer)->property_text() =
				Glib::ustring::format(std::setw(2), track->TrackNumber(), ". ", track->Name());
		else
			((Gtk::CellRendererText*)renderer)->property_text() = mtp_data->Name();
	}
	else
		((Gtk::CellRendererText*)renderer)->property_text() = mtp_data->Name();
}

// Add the albums and tracks to the store so they will be displayed.
void AlbumView::PopulateStore(const std::vector<Gtk::TreeIter>& album_list)
{
	for (std::vector<Gtk::TreeIter>::const_iterator album_iter = album_list.begin();
			(album_iter != album_list.end()); ++album_iter)
	{
		Glib::RefPtr<album_t> album =
			Glib::RefPtr<album_t>::cast_static((*album_iter)->get_value(m_StoreColumns.m_Data));

		Gtk::TreeModel::Row album_row = *(m_Store->append());
		album_row[m_StoreColumns.m_Data] = album;

		for (Gtk::TreeIter track_iter = (*album_iter)->children().begin();
				track_iter != (*album_iter)->children().end(); ++track_iter)
		{
			Gtk::TreeModel::Row track_row = *(m_Store->append(album_row.children()));
			track_row[m_StoreColumns.m_Data] = (*track_iter)->get_value(m_StoreColumns.m_Data);
		}
	}
}

int AlbumView::GetSelectedIndex()
{
	Gtk::TreeIter iter = get_selection()->get_selected();
	if (iter) return m_Store->get_path(iter)[0];
	else return -1;
}

AlbumSelector::AlbumSelector(const std::vector<Gtk::TreeIter>& album_list,
		const Glib::RefPtr<const track_t>& track)
	: Gtk::Dialog(_("Select Album For Track"), true),
	m_TrackName(_("Track:  ") + track->Name()),
	m_TrackArtist(_("Artist:  ") + track->Artist()),
	m_Description(_("The following albums were found with the same name as the track album. "
				"Select the one that you wish to add the track to.\n"
				"You may instead click new to create a new album for the track."))
{
	m_Description.set_line_wrap();
	get_content_area()->pack_start(m_TrackName, Gtk::PACK_SHRINK);
	get_content_area()->pack_start(m_TrackArtist, Gtk::PACK_SHRINK);
	get_content_area()->pack_start(m_Description, Gtk::PACK_SHRINK, 5);

	m_AlbumViewport.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	m_AlbumViewport.add(*Gtk::manage(m_AlbumView = new AlbumView));
	get_content_area()->pack_start(m_AlbumViewport);

	// Add the albums to the tree
	m_AlbumView->PopulateStore(album_list);

	// Dialog buttons
	add_button(Gtk::Stock::NEW, 1);
	add_button(Gtk::Stock::APPLY, Gtk::RESPONSE_APPLY);

	set_default_size(400, 400);

	show_all();
}

int AlbumSelector::Run(const std::vector<Gtk::TreeIter>& album_list,
		const Glib::RefPtr<const track_t>& track)
{
	AlbumSelector dialog(album_list, track);
	gint response = dialog.run();
	if (response == Gtk::RESPONSE_APPLY)
		return dialog.m_AlbumView->GetSelectedIndex();
	else
		return -1;
}
